import questionary
from sympy import sympify, pi

def ask_angle(prompt_text):
    """
    Asks the user for a single angle input and validates it.
    Supports expressions involving 'pi'.
    """
    while True:
        user_input = questionary.text(prompt_text).ask()
        try:
            # Try to parse and evaluate the user input as a sympy expression
            angle_expr = sympify(user_input, {"pi": pi})
            angle_val = float(angle_expr.evalf())
            return angle_val
        except Exception:
            print("Invalid angle input. Please enter a valid expression, e.g., pi/2, 3*pi/5, 1.5708")

def ask_sampling_time():
    """
    Asks the user for a sampling time T (in seconds) and validates it.
    """
    while True:
        user_input = questionary.text("Enter Sampling Time T in milliseconds (e.g. 200):").ask()
        try:
            T_val = float(user_input)
            if T_val <= 0:
                print("Sampling time must be greater than 0.")
            else:
                return T_val
        except ValueError:
            print("Invalid input. Please enter a numerical value.")

# Ask for four angles from the user
q1_0 = ask_angle("Enter q1_0 (e.g. 0, pi/2, -pi/3): ")
q2_0 = ask_angle("Enter q2_0 (e.g. -pi/2, pi/3): ")
q3_0 = ask_angle("Enter q3_0 (e.g. 0): ")
q4_0 = ask_angle("Enter q4_0 (e.g. pi/2): ")

# Predefined mode choices
mode_choice = questionary.select(
    "Select Optimization Mode:",
    choices=[
        "transparency",
        "stability"
    ]
).ask()

# Ask for sampling time from user input
T = ask_sampling_time()

# Determine the mass line based on the mode
if mode_choice == "transparency":
    mass_line = "m=M_eff*0.1; % effective mass of the HD (transparency)"
else:
    mass_line = "m=M_eff;     % effective mass of the HD (stability)"

matlab_code = f"""% User-defined initial joint configuration
ANGLES = [0 {q1_0} 0 pi/2 {q2_0} {q3_0} {q4_0}];

% This robot has 7 DOF. It is assumed that the 3rd joint is locked to avoid redundancy.
% It is assumed that the robot is simulating a virtual wall consisting of a
% virtual spring of Kw and virtual damping of Bw for a human operator, in
% different configurations (4 configurations).

% The following ANGLES are some predefined angles.

%% Defining dynamic parameters of KUKA LW4 robot
% --- do not change anything

d0=0.3100; d3=0.40; d5=0.39; d7=.078; db=0; 
b1=1.8; b2=1.6; b3=1.4; b4=1.2; b5=1; b6=0.8; b7=0.6;

B_Matrix=zeros(6,6);
B_Matrix(1,1)=b1;   B_Matrix(2,2)=b2;   B_Matrix(3,3)=b4;
B_Matrix(4,4)=b5;   B_Matrix(5,5)=b6;   B_Matrix(6,6)=b7;

E_s_7=[0;0;1;0;0;0];

q1_0=ANGLES(1); q2_0=ANGLES(2); q3_0=ANGLES(3);
q4_0=ANGLES(4); q5_0=ANGLES(5); q6_0=ANGLES(6); q7_0=ANGLES(7);

%% Here insert your original MATLAB code that calculates M, J_0, J_7, M_eff, B_eff, etc.
% Due to length, we omit rewriting every detail. Make sure to include:
% - the computations of M, J_0, R_0_7, J_7, M_eff, B_eff
% Then insert the chosen lines for mass_line and T.

% ...
% After computing M_eff and B_eff:
M_eff=E_s_7'*inv(J_7')*M*inv(J_7)*E_s_7;
Inv_J_7=inv(J_7);
B_eff=b1*Inv_J_7(1,3)^2 + b2*Inv_J_7(2,3)^2 + b4*Inv_J_7(3,3)^2 + b5*Inv_J_7(4,3)^2 + b6*Inv_J_7(5,3)^2 + b7*Inv_J_7(6,3)^2;

%% Equivalent 1-DOF haptic device
{mass_line}
b=B_eff*0.1; % effective viscous friction of the HD
T={T}/1000; % Sampling time (sec)
td=1*T; Td=td; % time delay in the control loop

%% Theoretical Stability Boundary
P1= -0.4087; P2=1.325; P3=-0.07507; q1=-pi/2;
a=m/b/(T+Td);K_H=0;
Omega_max=( -(P2-a*q1) - sqrt((P2-a*q1)^2-4*P3*(P1-a)))/(2*(P1-a))/(T+Td);
omega=linspace(0,Omega_max,10000);
Kw_U_T = (T*omega.*(K_H*sin((omega*(T + 2*Td))/2) - K_H*sin((omega*(3*T + 2*Td))/2) - b*omega.*cos((omega*(3*T + 2*Td))/2) - m*omega.^2.*sin((omega*(T + 2*Td))/2) + m*omega.^2.*sin((omega*(3*T + 2*Td))/2) + b*omega.*cos((omega*(T + 2*Td))/2)))./(4*cos((T*omega)/2) - 4*cos((T*omega)/2).^3);
Bw_U_T = -(T*(K_H*cos(Td*omega) - K_H*cos(omega*(T + Td)) - m*omega.^2.*cos(Td*omega) + b*omega.*sin(omega*(T + Td)) - b*omega.*sin(Td*omega) + m*omega.^2.*cos(omega*(T + Td))))./(4*cos((T*omega)/2) - 4*cos((T*omega)/2).^3);

T1=T+td; T2=T+td/2;
 
%%  Stability No Controller
BBW=linspace(0,20*b,20);
for jj=1:length(BBW)
	Bw=BBW(jj);
	K_max=max(Kw_U_T*2);
	K_min=0;
	for ii=1:30
		Kw=(K_min+K_max)/2;
	
		num=[1];
		den=[(m-Bw*T1) , b+Bw-Kw*T2 , Kw];
		
		% G=tf(num,den)
		[A,B,C,D]=tf2ss(num,den);
		
		R=real(eig(A));
		if R <0  % Stable
			K_min=Kw;
		else     % Unstable
			K_max=Kw;
		end
	end
	% Kw
	BB(jj)=Bw;
	KK(jj)=Kw;
end
BB=[0,BB,max(BB)];
KK=[0,KK,0];
fill(BB,KK,'r','FaceAlpha',0.3,'EdgeAlpha',.3)
grid on; hold on
text(0.8*max(BB),0.5*max(KK),'\bf Stable','BackgroundColor','white')



%%  Stability With controller
Q=[500 0 ; 0 500];
R=[1];
Bw=5*b;
Kw=(Bw+b)/(T/2+td);			% The LQR controller is going to be degined in the operating point of (Bw,Ww)=(5*b,(Bw+b)/(T/2+td))
num=[1];
den=[(m-Bw*T1) , b+Bw-Kw*T2 , Kw];
[A,B,C,D]=tf2ss(num,den);
[K,S,P] = lqr(A,B,Q,R);
K1=K(1);
K2=K(2);



BBW=linspace(0,20*b,20);
for jj=1:length(BBW)
	Bw=BBW(jj);
	K_max=max(Kw_U_T*2);
	K_min=0;
	for ii=1:30
		Kw=(K_min+K_max)/2;
	
		num=[1];
		den=[(m-Bw*T1) , b+Bw-Kw*T2 , Kw];
		
		% G=tf(num,den)
		[A,B,C,D]=tf2ss(num,den);
		
		R=real(eig(A-B*K));
		if R <0  % Stable
			K_min=Kw;
		else     % Unstable
			K_max=Kw;
		end
	end
	% Kw
	BB(jj)=Bw;
	KK(jj)=Kw;
end
BB=[0,BB,max(BB)];
KK=[0,KK,0];
fill(BB,KK,'b','FaceAlpha',0.3,'EdgeAlpha',.3)

% figure(1);plot(BB,KK,'b-'); grid on; hold on;

xlabel('\bf Virtual damping (Bw)')
ylabel('\bf Virtual stiffness (Kw)')
title('Stability region (z-width)')
legend('No Controller','With LQR Controller')
text(0.1*max(BB),0.9*max(KK),'\bf Unstable','BackgroundColor','white')
axis([0 max(BB) 0 max(KK)*1.1])


%%  Transparency @ (Bw,Kw)=(5*b,Kw=(Bw+b)/(T/2+td)) No Controller
Bw=5*b;
Kw=(Bw+b)/(T/2+td);	
NUM=[1];
DEN=[(m-Bw*T1) , b+Bw-Kw*T2 , Kw];
G_Close=tf(NUM,DEN);	% Closed-loop transfer function

s = tf("s");
G_des=1/(Kw+Bw*s);      % Desired Impedance
Z_trans=G_Close/G_des;	% Transparency transfer function
% Get the magnitude, phase, and frequency data
[mag, phase, wout] = bode(Z_trans,[0.01:0.01:100]);
figure(2); bode(Z_trans,[0.1:0.01:100]); grid on; hold on;


%% Transparency @ (Bw,Kw)=(5*b,10)	LQR Controller
Bw=5*b;
Kw=(Bw+b)/(T/2+td);	
num=[1];
den=[(m-Bw*T1) , b+Bw-Kw*T2 , Kw];
[A,B,C,D]=tf2ss(num,den);
[NUM,DEN]=ss2tf(A-B*K,B,C,D);
G_Close=tf(NUM,DEN);


s = tf("s");
G_des=1/(Kw+Bw*s);      % Desired Impedance
Z_trans=G_Close/G_des;	% Transparency transfer function


% Generate the Bode plot
figure(2);
[mag, phase, wout] = bode(Z_trans, [0.1:0.01:100]);
bode(Z_trans, [0.1:0.01:100], 'g--'); 
grid on; 
hold on;

% Find the axes handles
ax = findall(gcf, 'type', 'axes');

% Plot +3 dB and -3 dB lines on the magnitude plot
semilogx(ax(3), wout, 3*ones(1, length(wout)), 'r--'); % ax(3) is typically the magnitude plot
semilogx(ax(3), wout, -3*ones(1, length(wout)), 'r--'); 

% Add legends and titles
legend(ax(3), 'No Controller', 'With LQR Controller', '+3 dB Line', '-3 dB Line'); 
legend(ax(2), 'No Controller', 'With LQR Controller'); % For the phase plot
title(ax(3), '\bf Transparency Function');
"""

# Write the MATLAB code to a file
filename = "generated_code.m"
with open(filename, "w") as f:
    f.write(matlab_code)

print("\n--- MATLAB Code Generated ---\n")
print(f"Code has been saved to {filename}.")
print("\nYou can now open and run this file in MATLAB.")
