import os.path
import uuid

from ..muMLE.bootstrap.scd import bootstrap_scd
from ..muMLE.state.devstate import DevState
from ..muMLE.framework.conformance import Conformance, render_conformance_check_result
from ..muMLE.concrete_syntax.textual_od import parser
from ..muMLE.concrete_syntax.plantuml import renderer as plantuml

from ..muMLE.concrete_syntax.plantuml.make_url import make_url


class Model:

    def __init__(self, model: str, meta_model: uuid.UUID | None = None, is_file=True, state: DevState = None):
        self.state = DevState() if not state else state
        self.mm_model = bootstrap_scd(self.state) if not meta_model else meta_model
        self.m_model = self.parse(model, is_file)

    def get(self):
        return self.m_model

    @staticmethod
    def read_file(filename):
        cur_dir = os.path.dirname(__file__)
        with open(f"{cur_dir}/{filename}") as f:
            return f.read()

    def parse(self, model, is_file):
        mm_cs = self.read_file(model) if is_file else model
        return parser.parse_od(
            self.state, mm_cs, self.mm_model
        )

    def conform(self):
        conf = Conformance(self.state, self.m_model, self.mm_model)
        print(render_conformance_check_result(conf.check_nominal()))

    def get_uml(self, name: str, is_meta=False):
        if is_meta:
            plant_text = plantuml.render_package(name, plantuml.render_class_diagram(self.state, self.m_model))
        else:
            plant_text = plantuml.render_package(name, plantuml.render_object_diagram(self.state, self.m_model,
                                                                                      self.mm_model))

        print(make_url(plant_text))
