:- module(rdf_query_utils,
    [
        get_value_and_unit/4
    ]).

% Main function
get_value_and_unit(URI, QuantityType, Value, Unit) :-
    rdfs:rdf_has(URI, P1, O1),
    is_has_parameter_value(P1),

    % Step 2: Find the predicate isAttributedTo for O1
    rdfs:rdf_has(O1, P2, O2),
    is_is_attributed_to(P2),

    % Step 3: Find the predicate hasQuantityType for O2
    rdfs:rdf_has(O2, P3, QuantityTypeObject),
    is_has_quantity_type(P3),

    % Step 4: Check if the QuantityTypeObject is equal to the expected QuantityType
    is_correct_object_type(QuantityTypeObject, QuantityType),

    % Step 5: Retrieve the hasDoubleNumber and unit from O2
    rdfs:rdf_has(O1, P4, DoubleNumberObject),
    is_has_double_number(P4),
    convert_to_float(DoubleNumberObject, Value),

    rdfs:rdf_has(O1, P5, Unit),
    is_unit(P5).

% Helper function to convert the object to a float
convert_to_float(literal(type(_, Value)), ObjectFloat) :-
    % Convert the value to a string
    atom_string(Value, StringValue),
    % Convert the string to a float
    atom_number(StringValue, ObjectFloat).

% Default helper function to check if a Predicate is of a certain type
is_correct_predicate_type(Type, Predicate) :-
    % Ensure Predicate is atomic, otherwise error on is_instance(...) predicate
    atomic(Predicate),
    atom_concat(_, Type, Predicate).

% Helper function to check if Predicate is hasParameterValue
is_has_parameter_value(Predicate) :-
    is_correct_predicate_type('#hasParameterValue', Predicate).

% Helper function to check if Predicate is isAttributedTo
is_is_attributed_to(Predicate) :-
    is_correct_predicate_type('#isAttributedTo', Predicate).

% Helper function to check if Predicate is hasQuantityType
is_has_quantity_type(Predicate) :-
    is_correct_predicate_type('#hasQuantityType', Predicate).

% Helper function to check if Predicate is hasDoubleNumber
is_has_double_number(Predicate) :-
    is_correct_predicate_type('#hasDoubleNumber', Predicate).

% Helper function to check if Predicate is unit
is_unit(Predicate) :-
    is_correct_predicate_type('#unit', Predicate).

% Helper function to check if it is the correct QuantityType
is_correct_object_type(Object, Type) :-
    atom_concat(_, Type, Object).
