from uuid import UUID

from ..muMLE.api.od import ODAPI


def has_sus_as_target(get_od, sus: UUID, instrument: UUID):
    od: ODAPI = get_od()
    sus_uri = od.get_slot_value(sus, "uri")
    instrument_target_uri = od.get_slot_value(instrument, "target_uri")
    return sus_uri == instrument_target_uri


def __is_correct_connector(get_od, connector: UUID, port: str):
    od: ODAPI = get_od()
    return od.get_slot_value(connector, "port_name") == port


def is_negative_connector(get_od, connector: UUID):
    return __is_correct_connector(get_od, connector, "n")


def is_positive_connector(get_od, connector: UUID):
    return __is_correct_connector(get_od, connector, "p")


def __has_pattern(get_od, instrument: UUID, pattern: str):
    od: ODAPI = get_od()
    return od.get_slot_value(instrument, "pattern_uri") == pattern


def has_through_pattern(get_od, instrument: UUID):
    return __has_pattern(get_od, instrument, "through")


def has_across_pattern(get_od, instrument: UUID):
    return __has_pattern(get_od, instrument, "across")


def get_all_through_connections(get_od, sus_rc: UUID, port_name: str):
    od: ODAPI = get_od()
    connections = []

    next_rc = sus_rc
    while next_rc is not None:
        mutable_connection = od.get_outgoing(next_rc, "MutableConnection")
        if not mutable_connection:
            return connections

        target = od.get_target(mutable_connection[0])
        instrument = od.get_source(od.get_incoming(target, "ImmutableConnection")[0])
        connections.append(instrument)
        next_rc = [od.get_target(outgoing) for outgoing in od.get_outgoing(instrument, "ImmutableConnection") if
                   __is_correct_connector(get_od, od.get_target(outgoing), port_name)][0]

    return connections


def get_last_connected(get_od, sus_rc: UUID):
    od: ODAPI = get_od()
    has = od.get_outgoing(sus_rc, "HasAbstractConnection")[0]
    ac = od.get_target(has)
    connected_rc = od.get_source(ac)
    connected_ic = od.get_target(ac)
    return connected_rc, connected_ic


def delete_abstract_connection(get_od, sus_rc: UUID):
    od: ODAPI = get_od()
    has = od.get_outgoing(sus_rc, "HasAbstractConnection")[0]
    ac = od.get_target(has)
    od.delete(has)
    od.delete(ac)


def is_connected_to_ic(get_od, instrument_rc: UUID, sus_rc: UUID):
    od: ODAPI = get_od()
    ic = od.get_target(od.get_outgoing(sus_rc, "InstrumentnPoint")[0])
    for outgoing in od.get_outgoing(instrument_rc, "AbstractConnection"):
        target = od.get_target(outgoing)
        if target == ic:
            return True

    return False
