from uuid import UUID
from ..architecture.muMLE.api.od import ODAPI
from ..architecture.query.supplied_voltage import get_supplied_voltage


def get_one_port_from_rc(odapi: ODAPI, real_connector: UUID):
    return odapi.get_source(odapi.get_incoming(real_connector, "ImmutableConnection")[0])


def get_connection(odapi: ODAPI, mutable_connection: UUID):
    source = odapi.get_source(mutable_connection)
    target = odapi.get_target(mutable_connection)

    source_port = odapi.get_slot_value(source, "port_name")
    target_port = odapi.get_slot_value(target, "port_name")

    source_one_port = odapi.get_name(get_one_port_from_rc(odapi, source))
    target_one_port = odapi.get_name(get_one_port_from_rc(odapi, target))

    return (source_one_port, source_port), (target_one_port, target_port)


def get_voltage(odapi: ODAPI, instrument: UUID):
    result = get_supplied_voltage(odapi.get_slot_value(instrument, 'uri'))
    return result['Value'] if 'Value' in result else 0


components = {
    "Resistor": {
        "type": "Modelica.Electrical.Analog.Basic.Resistor",
        "parameters": {
            "R": "10",  # some sparql query function later
        }
    },
    "Battery": {
        "type": "Modelica.Electrical.Analog.Sources.TrapezoidVoltage",
        "parameters": {
            "nperiod": "1",
            "V": get_voltage,
            "rising": "1",
            "width": "5",
            "falling": "1",
            "period": "7"
        }
    },
    "Ammeter": {
        "type": "Modelica.Electrical.Analog.Sensors.CurrentSensor",
        "parameters": {}
    },
    "Ground": {
        "type": "Modelica.Electrical.Analog.Basic.Ground",
        "name": "ground"
    }
}
