import subprocess
import os
import sys


def run_gradle_task(task_name, project_directory):
    # Explicitly specify the path to 'gradlew'
    project_directory = os.path.abspath(project_directory)
    gradlew = os.path.join(project_directory, 'gradlew')

    if not os.path.exists(gradlew):
        print(f"gradlew script not found in {project_directory}")
        sys.exit(1)

    # Make 'gradlew' executable if it's not
    os.chmod(gradlew, 0o755)

    cmd = [gradlew, task_name]

    try:
        result = subprocess.run(
            cmd,
            cwd=project_directory,
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        print("Task completed successfully.")
        print(result.stdout.decode())
    except subprocess.CalledProcessError as e:
        print("Error running task.")
        print(e.stderr.decode())
