:- module(experiment_questions, [
    answer_type_for_question/2,
    options_for_question/2,
    next_questions/3,
    phase_of_question/2
]).

:- use_module(library(semweb/rdf_db)).

:- dynamic answer_type_for_question/2, options_for_question/2, next_questions/3.

% Register RDF prefixes
:- rdf_register_prefix(experiment, 'http://msdl.uantwerpen.be/experiment/vocabulary/core/experiment#').
:- rdf_register_prefix(instrument, 'http://msdl.uantwerpen.be/experiment/vocabulary/core/instrument#').
:- rdf_register_prefix(poi, 'http://msdl.uantwerpen.be/experiment/vocabulary/core/poi#').
:- rdf_register_prefix(resource, 'http://msdl.uantwerpen.be/experiment/vocabulary/core/resource#').
:- rdf_register_prefix(sus, 'http://msdl.uantwerpen.be/experiment/vocabulary/core/sus#').

% Q: Do you already have an experiment specification?
answer_type_for_question('Do you already have an experiment specification?', 'boolean').
options_for_question('Do you already have an experiment specification?', ['Yes', 'No']).
next_questions('Do you already have an experiment specification?', 'Yes', ['What is the experiment specification?']).
next_questions('Do you already have an experiment specification?', 'No', ['Is this an atomic experiment?']).

% Q: What is the experiment specification?
answer_type_for_question('What is the experiment specification?', 'radio').
options_for_question('What is the experiment specification?', Options) :-
    findall(SuS, rdf(SuS, rdf:type, sus:'System'), Options).

% Q: What is the time-base?
answer_type_for_question('What is the time base?', 'radio').
options_for_question('What is the time base?', ['Discrete time', 'Continous time', 'Now']).

% Q: What is SuS?
answer_type_for_question('What is the SuS?', 'text').
next_questions('What is the SuS?', SystemUnderStudy, ['What are the domains associated with the available interfaces to the selected SuS?']).

% Q: Will there be a stimulus?
answer_type_for_question('Will there be a stimulus?', 'boolean').
next_questions('Will there be a stimulus?', _Answer, ['Is this an atomic experiment?']).

% Q: Is this an atomic experiment?
answer_type_for_question('Is this an atomic experiment?', 'radio').
options_for_question('Is this an atomic experiment?', ['Yes', 'No']).
next_questions('Is this an atomic experiment?', _Answer, ['What is the SuS?']).


% Q5: What are the domains associated with the available interfaces to the selected SuS?
answer_type_for_question('What are the domains associated with the available interfaces to the selected SuS?', 'checkbox').
options_for_question('What are the domains associated with the available interfaces to the selected SuS?', ['Electrical', 'Thermal', 'Geometrical']).
next_questions('What are the domains associated with the available interfaces to the selected SuS?', SelectedDomains, NextQuestions) :-
    % Verifica se é uma lista ou uma string e normaliza para lista
    (   is_list(SelectedDomains) -> Domains = SelectedDomains
    ;   Domains = [SelectedDomains]
    ),
    findall(
        NextQuestion,
        (
            member(Domain, Domains),
            format(atom(NextQuestion), 'What are the quantities associated with the interfaces of the selected SuS in the ~w Domain?', [Domain])
        ),
        NextQuestions
    ).

% Q6: What are the quantities associated with the interfaces of the selected SuS in the X Domain?
answer_type_for_question(Question, 'checkbox') :-
    sub_atom(Question, 0, _, _, 'What are the quantities associated with the interfaces of the selected SuS in the '),
    sub_atom(Question, _, _, 0, ' Domain?').

options_for_question(Question, ['Electric Current', 'Electric Potential Difference']) :-
    sub_atom(Question, _, _, 0, 'Electrical Domain?').

% For Thermal Domain
options_for_question(Question, ['Temperature']) :-
    sub_atom(Question, _, _, 0, 'Thermal Domain?').

% Next questions after quantities are selected
next_questions(Question, SelectedQuantities, NextQuestions) :-
    sub_atom(Question, 0, _, _, 'What are the quantities associated with the interfaces of the selected SuS in the '),
    sub_atom(Question, _, _, 0, ' Domain?'),
    % For each quantity, ask about interfaces
    findall(
        NextQuestion,
        (
            member(Quantity, SelectedQuantities),
            format(atom(NextQuestion), 'What are the interfaces of the selected SuS associated with ~w?', [Quantity])
        ),
        NextQuestions
    ).

% Q7 & Q8: What are the interfaces of the selected SuS associated with X?
answer_type_for_question(Question, 'checkbox') :- 
    sub_atom(Question, 0, _, _, 'What are the interfaces of the selected SuS associated with '), 
    sub_atom(Question, _, _, 0, '?').

options_for_question('What are the interfaces of the selected SuS associated with Electric Current?', ['UID:R1:curr_through_R1', 'UID:R1:potential_at_R1pinN']).
options_for_question('What are the interfaces of the selected SuS associated with Electric Potential Difference?', ['UID:R1:potential_across_R1', 'UID:R1:potential_at_R1pinP', 'UID:R1:potential_at_R1pinN']).

options_for_question('What are the interfaces of the selected SuS associated with Temperature?', ['UID:env:ambient_temp']).

% Next questions after interfaces are selected
next_questions(Question, SelectedInterfaces, NextQuestions) :-
    sub_atom(Question, 0, _, _, 'What are the interfaces of the selected SuS associated with '),
    % For each interface, ask about its nature
    findall(
        NextQuestion,
        (
            member(Interface, SelectedInterfaces),
            format(atom(NextQuestion), 'What is the nature, in the experiment, of interface ~w?', [Interface])
        ),
        NextQuestions
    ).

% Q9 & Q10 & Q14: What is the nature, in the experiment, of interface X?
answer_type_for_question(Question, 'checkbox') :-
    sub_atom(Question, 0, _, _, 'What is the nature, in the experiment, of interface '),
    sub_atom(Question, _, _, 0, '?').

options_for_question(Question, ['Stimulus (have control over)', 'Measurement']) :-
    sub_atom(Question, 0, _, _, 'What is the nature, in the experiment, of interface '),
    sub_atom(Question, _, _, 0, '?').

next_questions(Question, _Answer, NextQuestions) :-
    sub_atom(Question, 0, _, _, 'What is the nature, in the experiment, of interface '),
    % After nature is determined, proceed to next appropriate question
    NextQuestions = ['What are the possible types of properties of interest that can be calculated based on the selected interfaces?'].

% Q15: What are the possible types of properties of interest that can be calculated based on the selected interfaces?
answer_type_for_question('What are the possible types of properties of interest that can be calculated based on the selected interfaces?', 'checkbox').
options_for_question('What are the possible types of properties of interest that can be calculated based on the selected interfaces?', ['Current', 'Voltage', 'Temperature', 'Resistance', 'Power']).
next_questions('What are the possible types of properties of interest that can be calculated based on the selected interfaces?', _Answer, ['What kind of analysis is associated with the selected property of interest?']).

% Q24: What kind of analysis is associated with property of interest X?
answer_type_for_question('What kind of analysis is associated with the selected property of interest?', 'checkbox').
options_for_question('What kind of analysis is associated with the selected property of interest?', ['Steady-state', 'Transient']).
next_questions('What kind of analysis is associated with the selected property of interest?', _Answer, ['What is the transient period for the property in seconds?']).

% Q25: What is the transient period for the property in seconds?
answer_type_for_question('What is the transient period for the property in seconds?', 'number').
next_questions('What is the transient period for the property in seconds?', _Answer, ['What is the experimental architecture with the selected configuration?']).

% Q26: What is the experimental architecture with the selected configuration?
answer_type_for_question('What is the experimental architecture with the selected configuration?', 'text').
next_questions('What is the experimental architecture with the selected configuration?', _Answer, ['Where is the architecture stored in memory?']).

% Q27: Where is the architecture stored in memory?
answer_type_for_question('Where is the architecture stored in memory?', 'text').
next_questions('Where is the architecture stored in memory?', _Answer, ['What language is the architecture expressed in?']).

% Q28: What language is the architecture expressed in?
answer_type_for_question('What language is the architecture expressed in?', 'radio').
options_for_question('What language is the architecture expressed in?', ['Modelica', 'Simulink', 'VHDL-AMS']).
next_questions('What language is the architecture expressed in?', _Answer, ['What is the workflow with the selected configuration?']).

% Q29: What is the workflow with the selected configuration?
answer_type_for_question('What is the workflow with the selected configuration?', 'text').
next_questions('What is the workflow with the selected configuration?', _Answer, ['Which activities are in the workflow with the selected configuration?']).

% Q30: Which activities are in the workflow with the selected configuration?
answer_type_for_question('Which activities are in the workflow with the selected configuration?', 'checkbox').
options_for_question('Which activities are in the workflow with the selected configuration?', ['assemble_circuit', 'stimulate', 'measure']).
next_questions('Which activities are in the workflow with the selected configuration?', _Answer, ['Who is the actor for assemble_circuit?']).

% Q31: Who is the actor for assemble_circuit?
answer_type_for_question('Who is the actor for assemble_circuit?', 'checkbox').
options_for_question('Who is the actor for assemble_circuit?', ['Rakshit Mittal', 'Lucas Lima', 'Raheleh Eslampanah']).
next_questions('Who is the actor for assemble_circuit?', _Answer, []).

% Default options_for_question when no options are needed
options_for_question(_, []).

% Default next_questions when no next questions
next_questions(_, _, []).

% Phase 1
phase_of_question('Do you already have an experiment specification?', 1).
phase_of_question('What is the experiment specification?', 1).

% Phase 2
phase_of_question('What is the time base?', 2).
phase_of_question('What is the SuS?', 2).
phase_of_question('Will there be a stimulus?', 2).
phase_of_question('Is this an atomic experiment?', 2).

% Phase 3
phase_of_question('What are the domains associated with the available interfaces to the selected SuS?', 3).
phase_of_question(Question, 3) :-
    sub_atom(Question, 0, _, _, 'What are the quantities associated with the interfaces of the selected SuS in the'),
    !;
    sub_atom(Question, 0, _, _, 'What are the interfaces of the selected SuS associated with'),
    !;
    sub_atom(Question, 0, _, _, 'What is the nature, in the experiment, of interface'),
    !.

% Phase 4
phase_of_question('What are the possible types of properties of interest that can be calculated based on the selected interfaces?', 4).
phase_of_question('What kind of analysis is associated with the selected property of interest?', 4).
phase_of_question('What is the transient period for the property in seconds?', 4).
phase_of_question('What is the experimental architecture with the selected configuration?', 4).
phase_of_question('Where is the architecture stored in memory?', 4).
phase_of_question('What language is the architecture expressed in?', 4).
phase_of_question('What is the workflow with the selected configuration?', 4).
phase_of_question('Which activities are in the workflow with the selected configuration?', 4).
phase_of_question('Who is the actor for assemble_circuit?', 4).

phase_of_question(_, -1).
