from janus_swi import *


def add_owl_class(class_name):
    """
    Add an OWL class to the ontology.
    """
    query_once(f"semweb:rdf_db:rdf_assert({class_name}, rdf:type, owl:'Class')")
    print(f"Added OWL class: {class_name}")


def add_owl_individual(individual, class_name):
    """
    Add an individual as an instance of an OWL class.
    """
    query_once(f"semweb:rdf_db:rdf_assert({individual}, rdf:type, {class_name})")
    print(f"Added individual: {individual} to class: {class_name}")


def add_owl_object_property(subject, property_name, object_):
    """
    Add an OWL object property (relationship between two individuals).
    """
    query_once(f"rdf_assert({property_name}, rdf:type, owl:'ObjectProperty'), rdf_assert({subject}, {property_name}, {object_})")
    print(f"Added object property: {subject} --[{property_name}]--> {object_}")


def query_owl_class_members(class_uri):
    """
    Query all individuals that are members of a given OWL class.
    """
    result = []
    with query(f"semweb:rdfs:rdfs_individual_of(I,'{class_uri}')") as q:
        while s := q.next():
            result.append(s['I'])
    return result


def query_owl_property(subject, property_name):
    """
    Query triples involving a given object property.
    """
    result = []
    with query(f"rdf({subject}, {property_name}, {object_})") as q:
        while s := q.next():
            result.append(s['I'])
    return result
