import inquirer


class Question:
    """
    Question class.
    """
    def __init__(self, qstring: str, qtype: str, implication_function, option_list = None, **kwargs):
        self.qstring = qstring
        self.qtype = qtype
        self.option_list = option_list
        self.implication_function = implication_function
        self.kwargs: dict = kwargs

    def __call__(self):
        """
        Calling a question based on question type.
        """
        if self.qtype == 'str':
            answer: str = inquirer.prompt([inquirer.Text(name='value', message=self.qstring)])['value']
            self.implication_function(answer, **self.kwargs)
            return answer
        elif self.qtype == 'bool':
            answer: bool = inquirer.prompt([inquirer.Confirm(name='value', message=self.qstring)])['value']
            self.implication_function(answer, **self.kwargs)
            return answer
        elif self.qtype == 'int':
            answer: int = int(inquirer.prompt([inquirer.Text(name='value', message=self.qstring)])['value'])
            self.implication_function(answer, **self.kwargs)
            return answer
        elif self.qtype == 'float':
            answer: float = float(inquirer.prompt([inquirer.Text(name='value', message=self.qstring)])['value'])
            self.implication_function(answer, **self.kwargs)
            return answer
        elif self.qtype == 'single-choice':
            answer = inquirer.prompt([inquirer.List(name='value', message=self.qstring, choices=self.option_list)])['value']
            self.implication_function(answer, **self.kwargs)
            return answer
        elif self.qtype == 'multi-choice':
            answer = inquirer.prompt([inquirer.Checkbox(name='value', message=self.qstring, choices=self.option_list)])['value']
            self.implication_function(answer, **self.kwargs)
            return answer
        else:
            raise RuntimeError('The qtype for this question is not implemented')