:- module(questionnaire, [
       ask_number_question/2,
       ask_text_question/2,
       ask_radio_question/3,
       ask_none_radio_question/3,
       ask_checkbox_question/3,
       ask_boolean_question/2
   ]).

% Wrapper to ask a number question
ask_number_question(Question, Answer) :-
    writeln(Question),
    read(Answer),
    integer(Answer).

% Wrapper to ask a text input question
ask_text_question(Question, Answer) :-
    writeln(Question),
    read_line_to_string(user_input, Answer).

% Wrapper to ask a radio button (single choice) question
ask_radio_question(Question, Options, Answer) :-
    py_call(questionnaire:ask_radio, X), py_call(X:'__call__'(Question, Options), Answer).

% Wrapper to ask a checkbox (multiple choice) question
ask_checkbox_question(Question, Options, Answers) :-
    py_call(questionnaire:ask_checkbox, X), py_call(X:'__call__'(Question, Options), Answers).

% Wrapper to ask a boolean question (returns true for 'yes' and false for 'no')
ask_boolean_question(Question, Answer) :-
    py_call(questionnaire:ask_checkbox, X), py_call(X:'__call__'(Question), BoolResponse),
    (BoolResponse = 'yes' -> Answer = true ; Answer = false).

% Wrapper to ask a radio button question with an added "None" option
ask_none_radio_question(Question, Options, Answer) :-
    append(Options, ['None'], NewOptions),  % Add "None" to the options
    ask_radio_question(Question, NewOptions, Answer).