import { Box, createTheme, ThemeProvider } from '@mui/material';
import { BrowserRouter as Router, Routes, Route } from 'react-router-dom';
import Sidebar from './components/Sidebar';
import Questionnaire from './components/Questionnaire';
import Wee from './components/Wee';
import { useState, useEffect } from 'react';
import CssBaseline from '@mui/material/CssBaseline';
import Fuseki from './components/Fuseki';

const App = () => {
  const initialTheme = localStorage.getItem('theme') === 'dark';

  const [darkMode, setDarkMode] = useState(initialTheme);

  useEffect(() => {
    if (darkMode) {
      document.body.classList.add('dark');
      localStorage.setItem('theme', 'dark');
    } else {
      document.body.classList.remove('dark');
      localStorage.setItem('theme', 'light');
    }
  }, [darkMode]);

  const lightTheme = createTheme({
    palette: {
      mode: 'light',
      primary: {
        main: 'rgb(0,196,167)', 
      },
      secondary: {
        main: '#f50057',
      },
      text: {
        primary: '#000',
        secondary: '#555', 
      },
    },
  });

  const darkTheme = createTheme({
    palette: {
      mode: 'dark',
      primary: {
        main: 'rgb(0,196,167)', 
      },
      secondary: {
        main: '#f50057',
      },
      text: {
        primary: '#fff',
        secondary: 'rgba(255,255,255,0.7)',
      },
    },
    components: {
      MuiCheckbox: {
        styleOverrides: {
          root: {
            color: 'rgb(0,196,167)',
            '&.Mui-checked': {
              color: 'rgb(0,196,167)', 
            },
          },
        },
      },
      MuiRadio: {
        styleOverrides: {
          root: {
            color: 'rgb(0,196,167)',
            '&.Mui-checked': {
              color: 'rgb(0,196,167)', 
            },
          },
        },
      },
      MuiSelect: {
        styleOverrides: {
          icon: {
            color: 'rgb(0,196,167)', 
          },
        },
      },
      MuiFormControlLabel: {
        styleOverrides: {
          label: {
            color: darkMode ? 'rgb(255,255,255)' : 'rgb(0,0,0)',
          },
        },
      },
    },
  });

  const toggleTheme = () => {
    setDarkMode(!darkMode);
  };

  return (
    <Router>
      <ThemeProvider theme={darkMode ? darkTheme : lightTheme}>
        <CssBaseline />
        <Box className="flex">
          <Sidebar toggleTheme={toggleTheme} darkMode={darkMode} />
          <Box className="flex-grow">
            <Routes>
              <Route path="/" element={<Questionnaire />} />
              <Route path="/wee" element={<Wee />} />
              <Route path="/fuseki" element={<Fuseki />} />
            </Routes>
          </Box>
        </Box>
      </ThemeProvider>
    </Router>
  );
};

export default App;
