import  { useEffect, useState } from 'react';
import Box from '@mui/material/Box';
import LinearProgress from '@mui/material/LinearProgress';

type LinearDeterminateProps = {
  onComplete: () => void;
  reset: boolean;
};

export default function LinearDeterminate({ onComplete, reset }: LinearDeterminateProps) {
  const [progress, setProgress] = useState(0);

  useEffect(() => {
    if (reset) {
      setProgress(0);
    }

    const timer = setInterval(() => {
      setProgress((oldProgress) => {
        const diff = Math.random() * 20;
        const newProgress = Math.min(oldProgress + diff, 100);

        if (newProgress === 100) {
          clearInterval(timer);
          onComplete();
        }

        return newProgress;
      });
    }, 500);

    return () => {
      clearInterval(timer);
    };
  }, [onComplete, reset]);

  return (
    <Box sx={{ width: '100%' }}>
      <LinearProgress variant="determinate" value={progress} />
    </Box>
  );
}
