import { useState } from 'react';
import Typography from '@mui/material/Typography';
import LinearDeterminate from './LinearProgress';
import { Button } from '@mui/material';

export default function LoadingWorflow() {
  const [phase, setPhase] = useState(1);

  const handleProgressComplete = () => {
    if (phase === 1) {
      setPhase(2);
      setTimeout(() => {
        setPhase(3);
      }, 3000);
    } else if (phase === 3) {
      setPhase(4);
    }
  };

  const renderContent = () => {
    switch (phase) {
      case 1:
        return "Searching for an experiment...";
      case 2:
        return "Couldn't find an experiment matching this goal";
      case 3:
        return "Generating experiment architecture and workflow...";
      case 4:
        return (
          <>
            <Typography style={{ marginBottom: '1rem' }} variant="h4">Generate experiment with success</Typography>
            <div className='flex flex-row w-full justify-between'>
            <Button
            sx={{width: '45%'}}
              variant='contained'
              color='primary'
              href='./download/R1_ArchC1.drawio'
              download
            >
              Download Architecture
            </Button>
            <Button
            sx={{width: '45%'}}
              variant='contained'
              color='primary'
              href='./download/Experiment1-AssembleComponents.drawio'
              download
            >
              Download Workflow
            </Button>
            </div>
          </>
        );
      default:
        return "Loading...";
    }
  };

  return (
    <div className='flex flex-col px-4 py-8 bg-light-green dark:bg-light-black items-center shadow-2xl justify-center w-3/5 rounded-lg text-black dark:text-white'>
      <Typography sx={{ marginBottom: '1rem' }} id="modal-modal-title" variant="h5">
        {renderContent()}
      </Typography>
      {(phase === 1 || phase === 3) && (
        <LinearDeterminate onComplete={handleProgressComplete} reset={phase === 3} />
      )}
      {phase === 4 && (
        <Button sx={{ marginTop: '1rem', backgroundColor: 'var(--alert)'}} variant='contained' onClick={() => window.location.reload()}>
          Restart Quest
        </Button>
      )}
    </div>
  );
}
