export type Question = {
  question: string;
  options?: string[];
  type: 'text' | 'select' | 'checkbox' | 'radio';
  phase: number;
  flux?: Record<string, Question[] | string>;
};

export const phase1: Question[] = [
  {
    question: "Is this an atomic experiment? (an atomic experiment does not have repetitions or composition of sub-experiments)",
    options: ["Yes", "No"],
    type: 'radio',
    phase: 1,
    flux: {
      "Yes": 'phase2',
      "No": 'notAtomicyQuestions'
    }
  },
];

// Fase 2
export const phase2: Question[] = [
  { 
    question: "What is SuS?", 
    options: ["UID:R1"], 
    type: 'select', 
    phase: 2 
  },
  { 
    question: "What are the domains associated with the available interfaces to UID:R1?", 
    options: ["Electrical", "Thermal", "Geometrical"], 
    type: 'checkbox', 
    phase: 2,
    flux: {
      "Electrical": 'phase2Electrical',
      "Thermal": 'phase2Thermal',
      "Geometrical": 'phase2Geometrical'
    }
  },
];

export const phase2Electrical: Question[] = [
  { 
    question: "What are the quantities associated with the interfaces of UID:R1 in the Electrical Domain?", 
    options: ["Electric Current", "Electric Potential Difference (will explore both)"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Electric Current?", 
    options: ["UID:R1:curr_through_R1"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Electric Potential Difference?", 
    options: ["UID:R1:potential_across_R1", "UID:R1:potential_at_R1pinP", "UID:R1:potential_at_R1pinN"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:curr_through_R1 ?", 
    options: ["Stimulus", "Measurement"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:potential_across_R1 ?", 
    options: ["Stimulus (have control over)", "Measurement"], 
    type: 'checkbox', 
    phase: 2,
    flux: {
      "Stimulus (have control over)": 'phase3',
      "Measurement": 'phase3'
    }
  }
];

export const phase2Thermal: Question[] = [
  { 
    question: "What are the quantities associated with the interfaces of UID:R1 in the Thermal Domain?", 
    options: ["Heat Transfer", "Temperature"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Heat Transfer?", 
    options: ["UID:R1:heat_through_R1"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Temperature?", 
    options: ["UID:R1:temperature_at_R1_surface", "UID:R1:temperature_at_R1_internal"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:heat_through_R1?", 
    options: ["Stimulus", "Measurement"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:temperature_at_R1_surface?", 
    options: ["Stimulus (have control over)", "Measurement"], 
    type: 'checkbox', 
    phase: 2,
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:temperature_at_R1_internal?", 
    options: ["Stimulus (have control over)", "Measurement"], 
    type: 'checkbox', 
    phase: 2,
    flux: {
      "Stimulus (have control over)": 'phase3',
      "Measurement": 'phase3'
    }
  }
];

export const phase2Geometrical: Question[] = [
  { 
    question: "What are the quantities associated with the interfaces of UID:R1 in the Geometrical Domain?", 
    options: ["Position", "Displacement", "Orientation"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Position?", 
    options: ["UID:R1:position_at_R1"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Displacement?", 
    options: ["UID:R1:displacement_of_R1"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What are the interfaces of UID:R1 associated with Orientation?", 
    options: ["UID:R1:orientation_of_R1"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:position_at_R1?", 
    options: ["Stimulus", "Measurement"], 
    type: 'checkbox', 
    phase: 2 
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:displacement_of_R1?", 
    options: ["Stimulus (have control over)", "Measurement"], 
    type: 'checkbox', 
    phase: 2,
  },
  { 
    question: "What is the nature, in the experiment, of interface UID:R1:orientation_of_R1?", 
    options: ["Stimulus (have control over)", "Measurement"], 
    type: 'checkbox', 
    phase: 2,
    flux: {
      "Stimulus (have control over)": 'phase3',
      "Measurement": 'phase3'
    }
  }
];

export const phase3: Question[] = [
  { question: "What are the domains associated with the available interfaces to Environment?", options: ["Electromagnetic", "Thermal"], type: 'checkbox', phase: 3  },
  { question: "What are the quantities associated with the interfaces of Environment in the Thermal Domain?", options: ["Temperature"], type: 'checkbox', phase: 3  },
  { question: "What are the interfaces of the Environment associated with Temperature?", options: ["UID:env:ambient_temp"], type: 'checkbox', phase: 3  },
  { question: "What is the nature, in the experiment, of interface UID:env:ambient_temp ?", options: ["Stimulus (have control over)", "Measurement"], type: 'checkbox', phase: 3,
    flux: {
      "Stimulus (have control over)": 'phase4',
      "Measurement": 'phase4'
    }
  }
]

export const phase4: Question[] = [
  { question: "What are the possible types of properties of interest that can be calculated based on the selected interfaces UID:R1:potential_across_R1, UID:R1:curr_through_R1, UID:env:ambient_temp?", options: ["Current", "Voltage", "Temperature", "Resistance", "Power", "…"], type: 'checkbox', phase: 4  },
  { question: "What is the equation (in general, a function of all selected interfaces) associated with the calculation of Current (I_calculated)?", options: ["Eq2: I_calculated = I_measured"], type: 'checkbox', phase: 4  },
  { question: "Which interface realises I_measured in I_calculated = I_measured?", options: ["UID:R1:curr_through_R1"], type: 'checkbox', phase: 4  },
  { question: "What is the time-base?",  options: ["Now", "Discrete time", "Continuous time"], type: 'select', phase: 4  },
  { question: "What are the parameters associated with UID:R1:potential_across_R1?", options: ["Supplied voltage", "Tolerance"], type: 'checkbox', phase: 4  },
  { question: "What is the value of parameter Supplied Voltage for UID:R1:potential_across_R1 in Volts?", type: 'text', phase: 4  },
  { question: "What is the value of parameter Tolerance for UID:R1:potential_across_R1 in Volts?", type: 'text', phase: 4  },
  { question: "What are the parameters associated with UID:R1:curr_through_R1?", options: ["Tolerance", "Expected Range"], type: 'checkbox', phase: 4  },
  { question: "What is the value of parameter Tolerance for UID:R1:curr_through_R1 in Amperes?", type: 'text', phase: 4  },
  { question: "What is the value of parameter Expected Range for UID:R1:curr_through_R1 in Amperes?", type: 'text', phase: 4  },
  { question: "What kind of analysis is associated with property of interest Current?", options: ["Steady-state", "Transient"], type: 'checkbox', phase: 4  },
  { question: "What is the transient period (unique for each property of interest) for the Current in seconds?", type: 'text', phase: 4  }
]


export const notAtomicyQuestions: Question[] = [
  { question: "Will the experiment be repeated?", options: ["Yes", "No"], type: 'radio', phase: 4 },
  { question: "How will the experiment be repeated?", options: ["with same parameters", "with randomly varying parameters", "with parameter sweep", "with parameter optimization"], type: 'checkbox', phase: 4 },
  { question: "What is the architecture of the experiment that will be repeated?", options: ["UID:ArchC1", "New (create new configuration)"], type: 'radio', phase: 4 },
  { question: "What is the workflow of the experiment that will be repeated?", options: ["UID:WF1C1", "New (create new configuration)"], type: 'radio', phase: 4 },
  { question: "How many times will the experiment be repeated?", type: 'text', phase: 4 },
  { question: "What are the types of property of interest that can be calculated based on the variable of interest of type, List:I, derived from collection of the property of interest I by repetition of the sub-experiment?", options: ["Plot", "Average", "Standard Distribution"], type: 'checkbox', phase: 4 },
  { question: "What is the property of interest of the experiment?", options: ["Plot", "Average", "Standard Distribution"], type: 'checkbox', phase: 4 }
];
